from __future__ import absolute_import
from unittest import TestCase
import numpy as np
from numpy import array
from .main import labels2contours


class TestSelect_labels2Contours(TestCase):
    def test_happy_flow(self):
        data = array(
            [
                [0, 5, 5, 0, 0, 0],
                [0, 5, 5, 0, 0, 0],
                [1, 1, 1, 0, 0, 0],
                [0, 2, 0, 0, 0, 0],
                [0, 2, 2, 0, 0, 0],
                [0, 2, 2, 2, 0, 0],
                [0, 2, 2, 2, 2, 0],
            ]
        )

        expected_output = [
            [[[0, 2]], [[2, 2]]], #1
            [[[1, 3]], [[1, 6]], [[4, 6]]], #2
            [[[1, 0]], [[1, 1]], [[2, 1]], [[2, 0]]], #5
        ]

        result = labels2contours(data)

        self.assertEqual(len(expected_output), len(result))

        for i in range(len(expected_output)):
            x = result[i]
            print(x)
            np.testing.assert_array_equal(x, expected_output[i])

    def test_deprated_warning(self):
        data = array(
            [
                [0, 5, 5, 0, 0, 0],
                [0, 5, 5, 0, 0, 0],
                [1, 1, 1, 0, 0, 0],
                [0, 2, 0, 0, 0, 0],
                [0, 2, 2, 0, 0, 0],
                [0, 2, 2, 2, 0, 0],
                [0, 2, 2, 2, 2, 0],
            ]
        )
        
        with self.assertRaises(DeprecationWarning):
            labels2contours(data, simplify_error=0.1)




from __future__ import absolute_import
from unittest import TestCase
from . import bbox_fitting_center
import numpy as np

class TestBbox_fitting(TestCase):
    def test_raise_warning_image(self):
        input_array = [
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]]
        with self.assertRaises(TypeError):
            bbox_fitting_center(input_array, [3, 3])

    def test_raise_warning_float_cor(self):
        input_array = np.array([
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]
        ])
        with self.assertRaises(ValueError):
            bbox_fitting_center(input_array, [3.9, 3])
        with self.assertRaises(ValueError):
            bbox_fitting_center(input_array, [3, 3.0])

    def test_crop_5x5_3x3(self):
        input_array = np.array([
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]
        ])

        expected = np.array([
            [1, 1, 1],
            [1, 1, 2],
            [1, 1, 3],
        ])

        result = bbox_fitting_center(input_array, (3, 3))
        np.testing.assert_array_equal(expected, result)

    def test_crop_5x5_5x3(self):
        input_array = np.array([
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]
        ])

        expected = np.array([
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
        ])

        result = bbox_fitting_center(input_array, (5, 3))
        np.testing.assert_array_equal(expected, result)

    def test_padding_and_cropping_5x5_7x3(self):
        input_array = np.array([
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]
        ])

        expected = np.array([
            [0, 1, 1, 1, 1, 1, 0],
            [0, 1, 1, 1, 2, 1, 0],
            [0, 1, 1, 1, 3, 1, 0],
        ])

        result = bbox_fitting_center(input_array, (7, 3))
        np.testing.assert_array_equal(expected, result)

    def test_padding_and_cropping_5x5_6x3(self):
        input_array = np.array([
            [0, 1, 1, 1, 0],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 2, 1],
            [1, 1, 1, 3, 1],
            [0, 1, 1, 1, 0]
        ])

        expected = np.array([
            [1, 1, 1, 1, 1, 0],
            [1, 1, 1, 2, 1, 0],
            [1, 1, 1, 3, 1, 0],
        ])

        result = bbox_fitting_center(input_array, (6, 3))
        np.testing.assert_array_equal(expected, result)

    def test_padding_and_cropping_4x4_6x3(self):
        input_array = np.array([
            [0, 1, 1, 1],
            [1, 1, 1, 1],
            [1, 1, 1, 2],
            [1, 1, 1, 3],
        ])

        expected = np.array([
            [0, 1, 1, 1, 1, 0],
            [0, 1, 1, 1, 2, 0],
            [0, 1, 1, 1, 3, 0],
        ])

        result = bbox_fitting_center(input_array, (6, 3))
        np.testing.assert_array_equal(expected, result)


import numpy as np
import cv2


def positions2contour(positions: np.ndarray, simplify_error: float = 0) -> np.ndarray:
    """Transforms a list of positions to opencv contour format

    Args:
        positions (np.ndarray): 2 dimensional array with shape [N, 2].
            For the N pixels of the shape given as x, y
        simplify_error (float, optional): Deprecated!!

    Raises:
        DeprecationWarning: simplification is no longer supported

    Returns:
        np.ndarray: Opencv contour
    """    
    max_x = np.max(positions[:, 0])
    min_x = np.min(positions[:, 0])
    max_y = np.max(positions[:, 1])
    min_y = np.min(positions[:, 1])

    positions -= np.array([min_x, min_y])
    transposed_positions = positions.transpose()

    array_mask = np.zeros((max_y + 1 - min_y, max_x + 1 - min_x), dtype="uint8")
    array_mask[transposed_positions[1], transposed_positions[0]] = 255

    contours = cv2.findContours(array_mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)[
        0
    ][0] + [min_x, min_y]

    if simplify_error > 0:
        raise DeprecationWarning('As of Tomni 1.10 simplify is no longer supported')

    return contours



from typing import List
from copy import deepcopy
import numpy as np

from ...transformers import json2dict
from .. import add_area, add_circularity


def summary_json(
    json_list: List[dict], keyword: str, do_copy: bool = True, rounding: int = 2
) -> tuple:
    """Makes a summary of the keyword variable inside of the json objects. Function works
       for the keyword 'area'. The summary consists of the number of objects,
       the total sum of keyword in json_list, the mean, maximum and average keyword in json_list.

    Args:
        json_list (List[dict]): a list of jsons in standard cytosmart forma
        keyword (str): 'area'
        do_copy (bool, optional): When true copies the json_list and does not change it.
                Defaults to True.
        rounding (int, optional): the number of decimals are used in the summary. Defaults to 2.

    Raises:
        ValueError: If a keyword is used that is not supported

    Returns:
        tuple: (Number of objects, Total sum of objects, Mean of objects, Max of objects, Min of objects)
    """
    if do_copy:
        copy_json_list = deepcopy(json_list)
    else:
        copy_json_list = json_list

    if keyword == "area":
        if not keyword in json_list[0]:
            for json_object in copy_json_list:
                add_area(json_object)
    else:
        raise ValueError(f"summary for {keyword} is not supported")

    keyword_list = json2dict(copy_json_list, [keyword])[keyword]

    return (
        len(keyword_list),  # Number of objects
        round(np.sum(keyword_list), rounding),  # Total sum of objects
        round(np.mean(keyword_list), rounding),  # Mean of objects
        round(np.max(keyword_list), rounding),  # Max of objects
        round(np.min(keyword_list), rounding),  # Min of objects
    )



from unittest import TestCase
from .main import flip_json



class TestFlipPatch(TestCase):
    def test_flips(self):
        input_json = {
                "type": "ellipse",
                "center": {"x": 9, "y": 3},
                "radiusX": 5,
                "radiusY": 5,
                "angleOfRotation": 0,
            }
        
        result_json = flip_json(input_json,10)
        expected_json = {
                "type": "ellipse",
                "center": {"x": 9, "y": 6},
                "radiusX": 5,
                "radiusY": 5,
                "angleOfRotation": 0,
            }

        
        self.assertDictEqual(result_json, expected_json)

    def test_flips_polygon(self):
        input_json = {
                "type": "polygon",
                "points": [{"x": 7, "y": 2}, {"x": 7, "y": 6}, {"x": 12, "y": 5}],
            }
        result_json = flip_json(input_json,10)
        expected_json = {
                "type": "polygon",
                "points": [{"x": 7, "y": 7}, {"x": 7, "y": 3}, {"x": 12, "y": 4}],
            }
        self.assertDictEqual(result_json, expected_json)




from typing import Tuple
from typing import Tuple

import cv2
import numpy as np

def get_center(contour: np.ndarray) -> Tuple[int, int]:
    '''
    Return the center of a contour
    '''
    M = cv2.moments(contour)
    if M["m00"] == 0:
        return contour[0][0][0], contour[0][0][1]
    cX = int(M["m10"] / M["m00"])
    cY = int(M["m01"] / M["m00"])

    return cX, cY


from .positions2contour import positions2contour
from ..labels2listsOfPoints import labels2listsOfPoints
import numpy as np


def labels2contours(labels: np.ndarray, simplify_error: float = 0) -> list:
    '''
    Transforms an image with labels into opencv contours

    labels: (numpy.array) An array where every pixels is label to which object it belongs
    # simplify_error: (float) DEPRECATED the amount of error allowed well simplifying the contours
    '''
    listPoints = labels2listsOfPoints(labels)
    contours = [
        positions2contour(p, simplify_error=simplify_error)
        for p in listPoints
        if len(p) > 0
    ]
    return contours



import cv2
import math
from ...transformers import json2contours


def add_circularity(json_object: dict) -> None:
    """
    This function adds the property 'circularity' to the current json object.
    circularity is calculated by drawing a circle around the object,
    and then compare the area of the circle with area of the object.
    The bigger the difference between those 2 areas, the lower the circularity.

    json_object: (dict) a json object of type polygon
    """
    if json_object["type"] == "polygon":
        cnt = json2contours(json_object)

        # Calculate the area of enclosing circle
        enclosing_circle = cv2.minEnclosingCircle(cnt)
        radius = enclosing_circle[1]
        area_circle = radius ** 2 * math.pi

        # Calculate the area of contour
        area_contour = cv2.contourArea(cnt)
        circularity = area_contour / area_circle

        # Add results to current object
        json_object["circularity"] = circularity
    else:
        raise ValueError(
            f"currently only the polygon objects are supported not {json_object['type']}"
        )



from unittest import TestCase
from .main import rotate_json


class TestRotatePatch(TestCase):
    def test_rotation_expected_0(self):
        input_json = {
            "type": "ellipse",
            "center": {"x": 9, "y": 3},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        expected_json = {
            "type": "ellipse",
            "center": {"x": 9, "y": 3},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        true_json = rotate_json(input_json, 0, (5, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_0_polygon(self):
        input_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 1}, {"x": 1, "y": 5}],
        }

        expected_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 1}, {"x": 1, "y": 5}],
        }

        true_json = rotate_json(input_json, 0, (5, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_90(self):
        input_json = {
            "type": "ellipse",
            "center": {"x": 0, "y": 2},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        expected_json = {
            "type": "ellipse",
            "center": {"x": 3, "y": 0},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }
        true_json = rotate_json(input_json, 90, (6, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_90_polygon(self):

        input_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 1}, {"x": 1, "y": 5}],
        }

        expected_json = {
            "type": "polygon",
            "points": [{"x": 4, "y": 1}, {"x": 0, "y": 1}],
        }
        true_json = rotate_json(input_json, 90, (6, 5))

        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_180(self):
        input_json = {
            "type": "ellipse",
            "center": {"x": 0, "y": 2},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        expected_json = {
            "type": "ellipse",
            "center": {"x": 4, "y": 3},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        true_json = rotate_json(input_json, 180, (6, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_180_polygon(self):
        input_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 1}, {"x": 1, "y": 5}],
        }

        expected_json = {
            "type": "polygon",
            "points": [{"x": 3, "y": 4}, {"x": 3, "y": 0}],
        }
        true_json = rotate_json(input_json, 180, (6, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_270_polygon(self):
        input_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 1}, {"x": 1, "y": 4}],
        }

        expected_json = {
            "type": "polygon",
            "points": [{"x": 1, "y": 3}, {"x": 4, "y": 3}],
        }

        true_json = rotate_json(input_json, 270, (6, 5))

        self.assertDictEqual(true_json, expected_json)

    def test_rotation_expected_270(self):
        input_json = {
            "type": "ellipse",
            "center": {"x": 0, "y": 4},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        expected_json = {
            "type": "ellipse",
            "center": {"x": 4, "y": 4},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

        true_json = rotate_json(input_json, 270, (6, 5))
        self.assertDictEqual(true_json, expected_json)

    def test_angle_not_multiple_90(self):
        input_json = {
            "type": "ellipse",
            "center": {"x": 9, "y": 3},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }
        angle = 80
        self.assertRaises(ValueError, rotate_json, input_json, angle, (5, 5))



from ...transformers import json2contours
from ...contour_operations import get_center

def add_center(json_object: dict):


    if json_object['type']=='ellipse':
        pass
    elif json_object['type']=='polygon':
        contour = json2contours(json_object)
        x, y = get_center(contour)
        json_object['center'] = {"x": x, "y": y}
    else:
        raise ValueError(f"No json type found of {json_object['type']}")



import cv2
import numpy as np

from ..contours2json import contours2json
from ..labels2contours import labels2contours


def get_edges(mask: np.ndarray):
    """ get edges of the mask"""
    mask_copy = cv2.copyMakeBorder(
        mask,
        top=1,
        bottom=1,
        left=1,
        right=1,
        borderType=cv2.BORDER_CONSTANT,
        value=0,
    )
    edges = cv2.Canny(mask_copy, 50, 150)

    edges = cv2.dilate(edges, np.ones((5, 5)))

    edges = np.divide(edges, 255, dtype=np.float16)
    edges = edges.astype(np.uint8)
    edges = edges[1:-1, 1:-1]

    return edges


def mask2json(
    mask: np.ndarray, minimum_size_contours: int = 3, is_diagonal_connected=True
) -> list:
    """
    Converts binary mask to standard cytosmart format

    Args:
        mask (np.ndarray): binary mask
        minimum_size_contours (int, optional): The minimum number of points an contour should have to be included. Defaults to 3.
        is_diagonal_connected (bool, optional): If true diagonal pixels [[1, 0][0,1]] will be seen as the same object. Defaults to True.

    Returns:
        list: json_objects, e.g [{'type':'polygon', 'points':[{'x':1,'y':4},{'x':2,'y':3},{'x':4,'y':4} ]
    """
    connectivity = 4
    if is_diagonal_connected:
        connectivity = 8

    labels = cv2.connectedComponents(mask.astype(np.uint8), connectivity=connectivity)[
        1
    ]
    edges = get_edges(mask)

    edges = edges * labels
    del labels
    contours = labels2contours(edges)
    json_objects = contours2json(contours)
    json_objects = [
        json_object
        for i, json_object in enumerate(json_objects)
        if len(json_objects[i]["points"]) >= minimum_size_contours
    ]

    return json_objects



from typing import Union
import numpy as np
from ..contours2json import contours2json


def list_of_points2json(list_of_points: Union[list, np.array]) -> dict:
    ''''
    From a list of points in the form of [[x1, y1], [x2, y2], ... ,[xn, yn]] a json is made

    list_of_points: (list or numpy array) the list of points that discripe a polygon.
        In the form of [[x1, y1], [x2, y2], ... ,[xn, yn]].
    '''
    contours = np.array(list_of_points)
    contours = contours.reshape((
        1,
        contours.shape[0],
        1,
        contours.shape[1]
    ))

    json_object = contours2json(contours)[0]
    return json_object


import cv2
import numpy as np 

def bbox_fitting(img, x1, y1, x2, y2, padding_value=0):
    """
    Bounding box fitting:
    Using cropping and padding to fit an image to the bounding box.
    Relative to the 0,0 of the input image the image gets cropped and padded to fit with x1, y1, x2 and y2.
    The result is including the pixels x1, x2, y1 and y2.
    So a 0, 0, 9, 9 gives an 10x10 result.

    WARNING:
    Coordinates are IMAGE-coordinates. So that is the same pillow and OpenCV uses but swapped from what Numpy and Scipy use (ARRAY-coordinates).
    Yes, I hate it too.
    """

    if not isinstance(img, np.ndarray):
        raise TypeError("Img needs to be a numpy.ndarry not {}".format(type(img)))
    
    if not isinstance(x1, int) or not isinstance(x2, int) or not isinstance(y1, int) or not isinstance(y2, int):
        raise ValueError("x1, y1, x2 and y2 need to be a positive int not {}, {}, {}, {}".format(type(x1), type(y1), type(x2), type(y2)))

    # if x1 < 0 or y1 < 0 or x2 < 0 or y2 < 0:
    #     raise ValueError("x1, y1, x2 and y2 needs to be a possitive int. Given: {}, {}, {}, {}".format(x1, y1, x2, y2))

    if x2 < 0:
        return np.zeros((y2 - y1, x2 -x1))
    if y2 < 0:
        return np.zeros((y2 - y1, x2 -x1))
    if x1 >= img.shape[1]:
        return np.zeros((y2 - y1, x2 -x1))
    if y1 >= img.shape[0]:
        return np.zeros((y2 - y1, x2 -x1))

    cy1 = max(0, y1)
    cy2 = min(img.shape[0], y2)
    cx1 = max(0, x1)
    cx2 = min(img.shape[1], x2)
    
    pad_left = - min(0, x1)
    pad_right = max(x2 - img.shape[1], 0)
    pad_top = - min(0, y1)
    pad_bottom = max(y2 - img.shape[0], 0)
    
    img = img[cy1:cy2, cx1:cx2]
    
    img = cv2.copyMakeBorder(img,
                            top=pad_top, 
                            bottom=pad_bottom,
                            left=pad_left, 
                            right=pad_right, 
                            borderType=cv2.BORDER_CONSTANT, 
                            value=padding_value)

    return img



from unittest import TestCase
import numpy as np
from .main import json2contours

class TestJson2contours(TestCase):
    def test_happy_flow(self):
        expectedResult  = np.array([[[500, 50]], [[500, 99]], [[539, 99]], [[539, 50]]])

        inputContours = {
            "type": "polygon",
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        result = json2contours(inputContours)

        np.testing.assert_equal(result, expectedResult)

    def test_value_error_unsupported_type(self):
        inputContours = {
            "type": "unicorn",
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        with self.assertRaises(ValueError):
            json2contours(inputContours)

    def test_value_error_no_type_given(self):
        inputContours = {
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        with self.assertRaises(ValueError):
            json2contours(inputContours)


from __future__ import division
import numpy as np
import cv2
from . import bbox_fitting_center


def make_mask_circle(img_shape: tuple, diameter: int) -> np.ndarray:
    """
    This function produces a boolean image with a circle of True surrounded by False.
    img_shape is in image-coordinates. Diameter is in pixels.
    In case the circle is bigger than 100 pixels a different function is used.
    The big circle function is less precise but better at memory management.

    img_shape: (tuple, shape (2,) ) the shape of the image given in image coordinates.
        These can be obtained with img_dim.
    diameter: (int) the diameter of the circle given in pixels.

    return:
        numpy array with type boolean
    """
    if diameter < 100:
        return make_small_mask_circle(img_shape, diameter)
    else:
        return make_big_mask_circle(img_shape, diameter)


def make_small_mask_circle(img_shape, diameter):
    xx, yy = np.mgrid[: img_shape[1], : img_shape[0]] * 2

    xx = xx.astype(np.int32)
    yy = yy.astype(np.int32)

    xx -= img_shape[1] - 1
    yy -= img_shape[0] - 1

    circle = xx ** 2 + yy ** 2
    kernel = circle <= diameter ** 2

    return kernel


def make_big_mask_circle(img_shape, diameter):
    out = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (diameter, diameter))
    out = bbox_fitting_center(out, img_shape)
    return out.astype(np.bool)



from unittest import TestCase
import numpy as np
from .main import add_center


class TestAddCenterJson(TestCase):
    def test_happy_flow(self):
        json_object = {
            "type": "polygon",
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        expected_result = {
            "type": "polygon",
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
            "center": {"x": 519, "y": 74},
        }

        add_center(json_object)

        self.assertDictEqual(json_object, expected_result)

    def test_single_point(self):
        json_object = {
            "type": "polygon",
            "points": [{"x": 500, "y": 50},],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        expected_result = {
            "type": "polygon",
            "points": [{"x": 500, "y": 50},],
            "id": "unicorn",
            "parents": [],
            "children": [],
            "center": {"x": 500, "y": 50},
        }

        add_center(json_object)

        self.assertDictEqual(json_object, expected_result)

    def test_circle(self):
        json_object = {
            "type": "ellipse",
            "center": {"x": 20, "y": 30},
            "radiusX": 10,
            "radiusY": 10,
            "angleOfRotation": 0,
            "id": "unicorn",
        }

        expected_result = {
            "type": "ellipse",
            "center": {"x": 20, "y": 30},
            "radiusX": 10,
            "radiusY": 10,
            "angleOfRotation": 0,
            "id": "unicorn",
        }

        add_center(json_object)

        self.assertDictEqual(json_object, expected_result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

with open("LICENSE") as license_file:
    license = license_file.read()

with open("requirements.txt") as requirements_file:
    requirements = requirements_file.read().split("\n")

setup_requirements = ["pytest-runner"]

test_requirements = ["pytest"]

setup(
    author="Tom Nijhof",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Tomni is a collection of image analysis functions usefull for CytoSmart solution.",
    entry_points={"console_scripts": ["tomni=tomni.cli:__init__"]},
    install_requires=requirements,
    long_description=readme + "\n\n" + history + "\n\n" + license,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="tomni",
    name="tomni",
    packages=find_packages(include=["tomni*"], exclude=["docs*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/cytosmart-bv/tomni",
    version="1.10.0",
    zip_safe=False,
)



import math
import cv2
from ...transformers import json2contours


def add_area(json_object: dict) -> None:
    """Takes in a json object (ellipse, polygon and circle, standard cytosmart format)
        add the objects area to the json object.

    Args:
        json_object (dict): following the standard CytoSMART format

    """
    type_anno = json_object["type"]

    if type_anno == "circle":
        area = math.pi * (json_object["radiusX"] ** 2)

    elif type_anno == "ellipse":
        area = math.pi * json_object["radiusX"] * json_object["radiusY"]

    elif type_anno == "polygon":
        contour = json2contours(json_object)
        area = cv2.contourArea(contour)
    else:
        raise ValueError(f"{type_anno} is not supported")

    json_object["area"] = area



from unittest import TestCase
from .main import scale_json


class TestScaleObject(TestCase):
    def test_scaling_ellipses(self):
        inputDict = {
            "type": "ellipse",
            "center": {"x": 20, "y": 30},
            "radiusX": 10,
            "radiusY": 10,
            "angleOfRotation": 0,
            "id": "unicorn",
        }
        inputScaling = 0.5

        expectedDict = {
            "type": "ellipse",
            "center": {"x": 10, "y": 15},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
            "id": "unicorn",
        }

        result = scale_json(inputDict, inputScaling)

        self.assertDictEqual(result, expectedDict)

    def test_scaling_polygon(self):
        inputDict = {
            "type": "polygon",
            "points": [
                {"x": 500, "y": 52},
                {"x": 500, "y": 100},
                {"x": 540, "y": 100},
                {"x": 540, "y": 52},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }
        inputScaling = 0.25

        expectedDict = {
            "type": "polygon",
            "points": [
                {"x": 125, "y": 13},
                {"x": 125, "y": 25},
                {"x": 135, "y": 25},
                {"x": 135, "y": 13},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        result = scale_json(inputDict, inputScaling)

        self.assertDictEqual(result, expectedDict)

    def test_unknown_type(self):
        inputDict = {
            "type": "unicornType",
        }
        inputScaling = 1.02
        with self.assertRaises(ValueError):
            result = scale_json(inputDict, inputScaling)




def translation_json(json_object: dict, x_translation: int, y_translation: int):
    """
    Takes in a json object (ellipse or polygon) and translates it
    json_object: (dict) following the standard format
    x_translation: (int) The translation in the x-direction
    y_translation: (int) The translation in the y-direction

    example:
    json_object = {
        "type": "ellipse", 
        "center": {"x": 100, "y": 200}, 
        "radiusX": 50, 
        "radiusY": 100, 
        "angleOfRotation": 0, 
        "id": "12345-abcde"}
    x_translation = 100
    y_translation = -20

    result = {
        "type": "ellipse", 
        "center": {"x": 200, "y": 180}, 
        "radiusX": 50, 
        "radiusY": 100, 
        "angleOfRotation": 0, 
        "id": "12345-abcde"}

    """
    newjson_object = json_object.copy()
    if json_object["type"] == "ellipse":
        newjson_object["center"]["x"] = int(
            round(json_object["center"]["x"] + x_translation)
        )
        newjson_object["center"]["y"] = int(
            round(json_object["center"]["y"] + y_translation)
        )

    elif json_object["type"] == "polygon":
        all_points = []
        for point in json_object["points"]:
            all_points.append(
                {
                    "x": int(round(point["x"] + x_translation)),
                    "y": int(round(point["y"] + y_translation)),
                }
            )
        newjson_object["points"] = all_points
    else:
        raise ValueError(f"type {json_object['type']} is not supported")

    return newjson_object




from typing import List


def json2vgg(
    json_list: List[dict],
    image_name: str,
    extension: str = ".jpg",
    add_accuracy: bool = True,
):
    """
    Transforms a a lists of jsons into a json that can be used in 'VGG Image Annotator'.
    json_objects: (list[dict]) the jsons are in the standard CytoSMART format.
    image_name: (string) the name of the image where the annotations will be overlayed with
                inside the 'VGG Image Annotator'.
    """
    file_name = f"{image_name}{extension}"
    regions = []

    for json_object in json_list:
        vgg_object = {"shape_attributes": {}}
        if json_object["type"] == "ellipse":
            vgg_object["shape_attributes"]["name"] = json_object["type"]
            vgg_object["shape_attributes"]["cx"] = json_object["center"]["x"]
            vgg_object["shape_attributes"]["cy"] = json_object["center"]["y"]
            vgg_object["shape_attributes"]["rx"] = json_object["radiusX"]
            vgg_object["shape_attributes"]["ry"] = json_object["radiusY"]
            vgg_object["shape_attributes"]["theta"] = json_object["angleOfRotation"]

        elif json_object["type"] == "polygon":
            vgg_object["shape_attributes"]["name"] = json_object["type"]
            vgg_object["shape_attributes"]["all_points_x"] = [
                f["x"] for f in json_object["points"]
            ]
            vgg_object["shape_attributes"]["all_points_y"] = [
                f["y"] for f in json_object["points"]
            ]
        else:
            raise ValueError(f"Object type {json_object['type']} not supported")
        if add_accuracy:
            vgg_object["region_attributes"] = {"accuracy": json_object["accuracy"]}
        else:
            vgg_object["region_attributes"] = {}
        regions.append(vgg_object)

    vgg_json = {
        file_name: {"filename": file_name, "regions": regions, "file_attributes": {}}
    }

    return vgg_json



#%%
import numpy as np 
from time import time

from tomni.select_labels.main import select_labels_as_indices
import pickle

path = r"C:\Users\tomni\Downloads\labels.dump"
fileObject = open(path,'rb')
data = pickle.load(fileObject)

# #%% version 3
# start = time()
# result = select_labels_as_indices(data)
# print("0.3.3 timing", time() - start)

#%% version 2
import numpy as np 
from time import time

from main_0_4 import select_labels_as_indices as select_labels_as_indices_v2

start = time()
result = select_labels_as_indices_v2(data)
print("scipy timing", time() - start)